<?php

namespace HCM\FPA\Calculos\Adiantamento;

use HCM\FPA\Calculos\Geral\Calculo;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaPtregraEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\FPA\Entity\FpaParamfolhahEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaFormcalculoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use HCM\FPA\Calculos\Geral\FGTS;
use HCM\FPA\Calculos\Geral\INSS;
use HCM\FPA\Calculos\Geral\IRRF;

class CalculoAdiantamento extends Calculo {

    public function calculaAdiantamento($vinculo, $tp_calc_mov, $contador){

        $calcula = $this->_inicializaVariaveis($vinculo, $tp_calc_mov);
        
        if(!$calcula){
            return;
        }
        
        if ($tp_calc_mov->retornaItensCalculo($vinculo) === false) {
            return;
        }
        
        try {
            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
            $this->_itens_calculo_principal =  $obj_tp_calc_mov->getInstancia();
            $this->_itens_calculo_principal->setNrTipoCalculo($this->_nr_tipo_calculo);
        } catch(Exception $e) {
            $message = $e->getMessage();
            $message .= ' - CALCULO.PHP';
            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
        }

        if ($this->_nr_tipo_calculo !== FpatipocalculoEntity::CALC_FECHAMENTO &&
            $this->_nr_tipo_calculo !== FpatipocalculoEntity::CALC_PROVISAO_CONTABIL){
            $liquido = $this->_controleLiquido();
            if ($liquido == true) {
                MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui liquido.', MsgCalculo::ERRO);
                return;
            }
        }

        $this->_controleLiquidoParcial();
        
        $this->_inicializaVariaveisRegra();

        $this->_calculo_regra->geraVariaveis();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_INICIO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );

        $this->_verificaTrabalhoEfetivo($vinculo);
        
        $nmvinculom = $vinculo->getNmvinculom();
        
        if($this->_dias_ferias > 0 && $this->_idPgAdiantFerias == 'N') {
           MsgCalculo::getInstance()->adicionaMensagem('Adiantamento não calculado para o vínculo '.$nmvinculom.' pois ele está de férias.', MsgCalculo::ERRO);
           return;
        }

        if($this->_dias_ferias > 0 && $this->_idPgAdiantFerias == 'S') {
            if($this->_dias_ferias > $this->_nrQtMaxDiasFerias) {
                MsgCalculo::getInstance()->adicionaMensagem('Adiantamento não calculado para o vínculo '.$nmvinculom.' pois ele está de férias.', MsgCalculo::ERRO);
                return;
            }
        }
        
        MemoriaCalculo::getInstance()->criaCabecalho($this->entityManager, $this->hcmParameters, $this->_vinculo,$this->_nr_tipo_calculo,$this->_sit_func_resumo,$this->_salario,$this->_dias_uteis,$this->_dias_rsr,
            $this->_mes_competencia, $this->_dt_pagamento, $this->_nrOcorrenciaCalculo,$this->_obj_tipo_calculo->getNmtipocalculo(),$this->_gozo_ferias);
        
        //Verifica desconto de liquido negativo maior qe o adiantamento
        if($this->dataSet['IDDESCSALDOADI'] == 'S'){
            $_mesCompetenciaAtual = DateUtil::getDataDeString($this->_mes_competencia);   
            $calc_folhaAtual = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                $this->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $_mesCompetenciaAtual, $_mesCompetenciaAtual
            )[0];
 
            $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
 
            $itemCalcCompetenciaAtual->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAtual->getNrcalculofolha());
 
            $evento_difSalarioLiquido = $this->_eventom->retornaEvento(760);
 
            $val = $itemCalcCompetenciaAtual->retornaValorPorEvento($evento_difSalarioLiquido, null);
 
            if ($val && ($this->_salario / 2) < $val){
                MsgCalculo::getInstance()->adicionaMensagem('Vínculo possuí saldo negativo maior que o adiantamento', MsgCalculo::ERRO);
                return;
            }
        }
        
        $this->_calculaRemuneracaoBase($vinculo, $this->_itens_calculo_principal, $this->_nr_tipo_calculo);
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_ANTES_CALC_ESPECIFICO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        $this->_executaAdiantamento($vinculo, $tp_calc_mov);
        
        $this->_geraPensao();
            
        $this->_calculaTotais();
        
        if($this->_nr_tipo_calculo!=FpatipocalculoEntity::CALC_13SAL) {
            $this->_calculo_regra->executaPontoRegra(
                $this->_idDebug, FpaPtregraEntity::PTREGRA_FINAL, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
            );
        }
        
        MemoriaCalculo::getInstance()->setMovimentos();
        MemoriaCalculo::getInstance()->setIncidencias();
        
        $arquivoMemoria = MemoriaCalculo::getInstance()->getNmArquivo();
        
        $lista = null;
        
        if($arquivoMemoria){
            $lista['ARQUIVOMEMORIA'] = FPA::getTempDirectory().'/'.$arquivoMemoria;
        }
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            return $lista;
        }
        if ($this->_simulacao === false) {
            $tp_calc_mov->persisteItensCalculo();
            return $lista;
        } else {
            $lista['SIMULACAO'] = $tp_calc_mov->retornaListaItensCalculo(
                $this->_mes_competencia, $this->_eventom, $vinculo
            );
            //Vinculo
            if(count($lista) > 0){
                $lista['SIMULACAO'][0][] = array(
                    'MESOCORRENCIA' => $vinculo->getNrvinculom(),
                    'TIPOEVENTO' => $vinculo->getNmvinculom(),
                    'NRFILHO' => $vinculo->getNrvinculom(),
                    'NREVENTOM' => 'Matricula: '.$vinculo->getCdmatricula(),
                    'NMEVENTOH' => 'Salario: ' .number_format($this->_salario, 2, ',','.'),
                    'UNIDREFEEVEN' => 'Admissão: '.substr($vinculo->getDtadmissaovinc(), 0, 10),
                    'VRREFFOLHA' => null,
                    'IDEVENDEMON' => null,
                    'VREVENTOFOLHA' => null,
                    'NRORDEIMPRES' => null,
                    'ORDEMTIPOEVENTO' => null,
                    'NRVINCULOM' => null,
                    'NRCALCULOFOLHAVINC' => null
                );    
            }
            
            return $lista;
        }
    }

    public function retrocederAdiantamento($vinculo, $tp_calc_mov){

        if ($tp_calc_mov->retornaItensCalculo($vinculo) === false) {
            MsgCalculo::getInstance()->adicionaMensagem('Não foram encontrados itens de calculo.');
        }

        $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
        $this->_itens_calculo_principal =  $obj_tp_calc_mov->getInstancia();
        
        MsgCalculo::getInstance()->setDados($this->_itens_calculo_principal->getNrcalculofolha(), $vinculo->getNrvinculom(), $this->_nrorg, 'ERROSCALCULO');

        $liquido = $this->_itens_calculo_principal->verificaLiquido(
            $this->_assoc_eventos, $this->_eventom, $this->_nrorg
        );
        $fechamento = $this->_itens_calculo_principal->verificaFechamento(
            $this->_assoc_eventos, $this->_eventom, $this->_nrorg
        );
        if ($liquido === true) {
            if ($fechamento === false) {
                $this->_retrocesso($this->_itens_calculo_principal);
                $this->_itens_calculo_principal->persist();
                if($this->_nr_tipo_calculo == FpatipocalculoEntity::CALC_FERIAS){
                    $this->_cancelaGozoFerias($vinculo,$this->_dtInicioGozo,$this->_dtFimGozo);
                }
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Vínculo Fechado e não foi retrocedido',MsgCalculo::ALERTA);
            }
        } else {
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo não tem líquido',MsgCalculo::ERRO);
        }
    }

    private function _executaAdiantamento($vinculo, $tp_calc_mov){

        if (in_array(FpaProcessoEntity::ADIANTAMENTO_SALARIO, $this->_listaProcessos) === false){
            return;
        }

        $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov(FpatipomovimentEntity::TP_MOV_MENSAL, -1));
        $this->_itens_adiantamento_ma =  $obj_tp_calc_mov->getInstancia();

        $this->_demonstra(
            $this->_itens_adiantamento_ma, FpatipoassoceveEntity::ASSOC_INCID_IRRF,
            FpatipoassoceveEntity::ASSOC_BASE_IRRF_CR
        );

        $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov(FpatipomovimentEntity::TP_MOV_MENSAL, -1));
        $this->_itens_adiantamento_ma =  $obj_tp_calc_mov->getInstancia();

        $this->_demonstra(
            $this->_itens_adiantamento_ma, FpatipoassoceveEntity::ASSOC_IRRF_CALC,
            FpatipoassoceveEntity::ASSOC_IRRF_JA_DESC_CALC
        );
        
        $evento = $this->_assoc_eventos->retornaEventoAssociado(
            FpatipoassoceveEntity::ASSOC_IRRF_JA_DESC_CALC, $this->_eventom, $this->_nrorg
        );
        $posicao = $this->_itens_calculo_principal->procuraEvento($evento, null);
        if ($posicao !== -1) {
            $this->_itens_calculo_principal->atribuiCalculoDemonstrativo(
                $posicao, FpaitecalcfolhaEntity::IDEVENDEMONCALC_SIM
            );
        }
        
        $dt_adm = $this->_vinculo->getDtadmissaovinc();
        $dt_comp = $this->_mes_competencia;
        if ($this->_idRecAdianPriMes == FpaParamfolhahEntity::IDRECADIANPRIMES_SIM) {
            $this->_geraReferenciaAdiantamento();
        } elseif ($dt_adm < $dt_comp) {
            $this->_geraReferenciaAdiantamento();
        }
        
        $this->_geraEventoRelacionado();
        
        if($this->_ultPagamento === FpaFormcalculoEntity::IDULTPGTOMES_S){
            $this->_calculaDiversos(
                $this->_itens_calculo_principal, null, FpatipoassoceveEntity::ASSOC_ADIANTAM_SAL_ULT_PAG_CR, 100,
                $this->_mes_competencia, 0.0
            );
        }else{
            $this->_calculaDiversos(
                $this->_itens_calculo_principal, null, FpatipoassoceveEntity::ASSOC_ADIANTAM_SAL_CR, 100,
                $this->_mes_competencia, 0.0
            );
        }

        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_CALC_ESPECIFICO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );

        if ($vinculo->getNrvinculoempreg() == GpevinculohEntity::VINCULO_EMPREGATICIO_MENOR_APRENDIZ) {
            $percentual = GpevinculohEntity::PERCENTUAL_FGTS_MENOR_APRENDIZ;
        } else{
            $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::FGTS);
        }
        FGTS::calcula(
            FpatipoincideEntity::INC_FGTS_ADIANT, FpatipoassoceveEntity::ASSOC_FGTS_ADIANTAMENTO,
            FpatipoassoceveEntity::ASSOC_INCID_FGTS_ADIANT, $this->_mes_competencia, $this->_itens_calculo_principal,
            $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
        );
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_FGTS, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        INSS::calcula(
            FpatipoincideEntity::INC_INSS_ADIANT, FpatipoassoceveEntity::ASSOC_INSS_ADIANTAMENTO,
            FpatipoassoceveEntity::ASSOC_INCID_INSS_ADIANT, $this->_mes_competencia, $this->_itens_calculo_principal,
            $this->_assoc_eventos, $this->_eventom, $this->_nrorg, null,null,null,0,false
        );
        

        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_INSS, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );

        IRRF::calcula(
            FpatipoincideEntity::INC_IRRF_ADIANT, FpatipoassoceveEntity::ASSOC_IRRF_ADIANTAMENTO,
            FpatipoassoceveEntity::ASSOC_INCID_IRRF_ADIANT, $this->_mes_competencia, $this->_itens_calculo_principal,
            $this->_assoc_eventos, $this->_eventom, $this->_valores_referencia, $vinculo, $this->_idIRAcumulaRecMinimo
        );

        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_IRRF, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
    }
    
    private function _geraReferenciaAdiantamento(){
        
        $gera_adiantamento = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_GER_ADIANT_MATER');
        
        $diasmaternidade = 0;
        if((int)$this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_MATERNIDADE]>0 && $gera_adiantamento == 1)
        {
            $diasmaternidade = $this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_MATERNIDADE];
        }
        if ($this->_dias_normais + $diasmaternidade >= $this->_nrDiasMinAdiantamento) {
            if($this->_ultPagamento === FpaFormcalculoEntity::IDULTPGTOMES_S){
                $evento_adiantamento = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_ADIANTAM_SAL_ULT_PAG_CR, $this->_eventom, $this->_nrorg
                );
            }else{
                $evento_adiantamento = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_ADIANTAM_SAL_CR, $this->_eventom, $this->_nrorg
                );
            }
            $this->_itens_calculo_principal->geraEvento($evento_adiantamento, null, 0, $this->_percentualAdiantamento, 1);
        }
    }
}
