<?php
namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\AlteSituacaoFuncionalRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\Util;
use Setforms2\Shared\ORM\DateTime;
use HCM\Util\DateUtil;
use Teknisa\Model\Gestaopessoas\Entities\GpeAltesitufunc;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use HCM\FPA\Repository\FeriasRepository;
use HCM\Geral\Services\ESocialService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use HCM\Geral\Entity\GpealtesitufuncEntity;
use Doctrine\DBAL\Connection;
use HCM\Geral\Services\EstruturasService;

class FeriasService {

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters, ESocialService $eSocialService, FeriasRepository $feriasRepository, EstruturasService $estruturasService) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->eSocialService = $eSocialService;
        $this->feriasRepository = $feriasRepository;
        $this->estruturasServices       = $estruturasService;
    }
    
    public function verificaPossibilidadeAlteracao($nrchave, $edicaoexclusao) {
        $edicaodisponivel = $this->eSocialService->verificaPossibilidadeAlteracao($nrchave, 'Gozo de Ferias', $edicaoexclusao);
         
        if ($edicaodisponivel === true){
            return true;
        } else {
            $message = Messages::evaluateMessage(MESSAGES::HCM00000_EDITAR_EXCLUIR_REGISTRO_AFASTAMENTO, $nrchave);
            return new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_WARNING);
        }
    }
    
    public function getAprovaFerias($parameters, $filters) {
      return $this->feriasRepository->getAprovaFerias($parameters, $filters);
    }
    
    public function enviaEmailReprova($nmvinculo, $estrutGeren, $nmgerem, $dtinicioprev1, $diasprev1, $dtfimprev1, $mensagem){
       
        $subject = "Notificação - Reprovação de Férias - Vínculo: ".$nmvinculo;
        
        $nmoperador=$this->hcmParameters->getNmoperador();
        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('cdoperador' => $this->hcmParameters->getCdoperador()));
        $emailOperadorLogado = $operador->getDsemailoper();
        $dados = $this->estruturasServices->retornaDadosResponsavelInformal($this->hcmParameters->getNrorgTrabalho(), $estrutGeren, $this->hcmParameters->getDtCompetencia());
        if(empty($dados)){
            return "e-mail do responsável pela entrutura Gerencial ".$estrutGeren." não cadastrado";
        }
        $email = $dados['CDCOMUNICAPARC'];
        $nmdestinatario = $dados['NMPESSOA'];
        $mail = $this->hcmParameters->returnParamsSmtp();
        $mensagem = $this->getMensagemReprova($nmoperador, $nmvinculo, $nmgerem, $nmdestinatario, $email, $dtinicioprev1, $diasprev1, $dtfimprev1, $mensagem);
        $return = $mail->newEmail($email, $mensagem, $subject, $nmdestinatario, $nmoperador, $emailOperadorLogado, null, null);
        
        //$email, $message, $subject, $nmoperador, $nmfrom, $from = null, $attachment = null, $oneAttachment = false
    }
    
    public function getMensagemReprova($nmoperador, $nmvinculo, $nmgerem, $nmrespgerem, $emailrespgerem, $dtinicioprev1, $diasprev1, $dtfimprev1, $mensagem){
        $conteudo =
        '<table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee" style="background-color: #eeeeee;">
           <tbody>
              <tr>
                 <td>
                    <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="0274BA" class="container590" style="background-color: #0274ba; text-align: center;">
                       <tbody>
                          <tr>
                             <td>
                                <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580">
                                   <tbody>
                                      <tr>
                                         <td align="center" valign="middle">
                                            <table border="0" align="middle" cellpadding="0" cellspacing="0" style="border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt;" class="">
                                               <tbody>
                                                  <tr>
                                                     <td align="center" valign="middle">
                                                        <p style="color: #ffffff; font-size: 25px; font-family: \'Questrial\', Helvetica, sans-serif; line-height: 25px; padding-top:35px;">NOTIFICAÇÃO</p>
                                                        <p style="color: #ffffff; font-size: 16px; font-family: \'Questrial\', Helvetica, sans-serif; line-height: 15px; padding-top:15px;"><br></p>
                                                     </td>
                                                  </tr>
                                               </tbody>
                                            </table>
                                         </td>
                                      </tr>
                                   </tbody>
                                </table>
                             </td>
                          </tr>
                       </tbody>
                    </table>
                 </td>
              </tr>
           </tbody>
        </table>
        <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee">
           <tbody>
              <tr>
                 <td align="center">
                    <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color">
                       <tbody>
                          <tr>
                             <td>
                                <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580">
                                   <tbody>
                                      <tr>
                                         <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: \'Questrial\', sans-serif; mso-line-height-rule: exactly; padding-bottom:15px;">
                                            <div>
                                               <p>Previsão de Férias reprovada pelo operador '.$nmoperador.' no dia '.date("d/m/Y").' às '.date("H:i").' para o vínculo '.$nmvinculo.' conforme informações abaixo:</p>
                                               <p><b>Unidade: </b>'.$nmgerem.'</p>
                                               <p><b>Responsável: </b>'.$nmrespgerem.'</p>
                                               <p><b>E-mail Responsável: </b>'.$emailrespgerem.'</p>
                                               <p><b>Vínculo: </b>'.$nmvinculo.'</p>
                                               <p><b>Início de Previsão 1: </b>'.$dtinicioprev1.'</p>
                                               <p><b>Qtde Dias Previsão 1: </b>'.$diasprev1.'</p>
                                               <p><b>Fim de Previsão 1: </b>'.$dtfimprev1.'</p>
                                               <p><b>Situação: </b><font style="color: red;"> Reprovado</font></p>
                                               <p>&nbsp;</p>
                                               <p><b>Observação: </b>'.$mensagem.'</p>
                                               
                                            </div>
                                         </td>
                                      </tr>
                                      <tr>
                                         <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td>
                                      </tr>
                                   </tbody>
                                </table>
                             </td>
                          </tr>
                       </tbody>
                    </table>
                 </td>
              </tr>
           </tbody>
        </table>';
        
        return $conteudo;
    }
} 