<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class PreFechamentoFgtsRepository {
    
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->conferenciaFgts = new ConferenciaFgts($entityManager);
        $this->informacoesSinteticasFgts = new InformacoesSinteticasFgts($entityManager);
        $this->informacoesSobreBasesCalculoFgts = new InformacoesSobreBasesCalculoFgts($entityManager);
    }
        
    public function getConferenciaFgts($parameters) {
        return $this->conferenciaFgts->getConferenciaFgts($parameters);
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        return $this->informacoesSinteticasFgts->getInformacoesSinteticasFgts($parameters);
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        return $this->informacoesSobreBasesCalculoFgts->getInformacoesSobreBasesCalculoFgts($parameters);
    }
}

class ConferenciaFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getConferenciaFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$conferenciaFgts);
        $sql->execute(
            array(
                'P_NRORG'          => $parameters['NRORG'],
                'P_MODALIDADECAL'  => 1,
                'P_IGNORAOCORR'    => 'N',
                'P_NRESTRUTGEREN'  => $parameters['NRESTRUTGEREN'],
                'P_OCORRECAL'      => 1,
                'P_COMPETENCIA'    => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL'  => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'      => '',
                'P_NRTABELA'       => $parameters['NRTABELA']
            )
        );
        return $sql->fetchAll();
    }
}

class InformacoesSinteticasFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$informacoesSinteticasFgts);
        $sql->execute(
            array(
                'P_NRORG'         => $parameters['NRORG'],
                'P_MODALIDADECAL' => 1,
                'P_IGNORAOCORR'   => 'N',
                'P_NRESTRUTGEREN' => $parameters['NRESTRUTGEREN'],
                'P_OCORRECAL'     => 1,
                'P_COMPETENCIA'   => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL' => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'     => '',
                'P_NRTABELA'      => $parameters['NRTABELA']
            )
        );
        return $sql->fetchAll();
    }
}

class InformacoesSobreBasesCalculoFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$informacoesSobreBasesCalculoFgts);
        $sql->execute(
            array(
                'P_NRORG'         => $parameters['NRORG'],
                'P_COMPETENCIA'   => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL' => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'     => '',
                'P_NRTABELA'      => $parameters['NRTABELA'],
                'P_MODALIDADECAL' => 1,
                'P_IGNORAOCORR'   => 'N',
                'P_OCORRECAL'     => 1
            )
        );
        return $sql->fetchAll();
    }
}

class Queries {
    
    static $conferenciaFgts = "
      SELECT * FROM ( 
        SELECT RES.VINCULO,
              RES.DTRESCISAOVINC,
              RES.MATRICULA,
              RES.REGISTRO,
              PE.NRCPFPESSOA,
              RES.NOME,
              RES.NRLEGAL,
              RES.NRGEREN,
              RES.RAZAOLEGAL,
              RES.CNPJLEGAL,
              RES.INTLEGAL,
              RES.RAZAOGEREN,
              RES.CNPJGEREN,
              RES.INTGEREN,
              ENVIO.IDSTATUS,
              ENVIO.NRTABELA,
              :P_COMPETENCIA PERIODO,
              SUBSTR(RES.CNPJLEGAL, 0,8) CNPJRAIZ,
              RES.FGTS BASE_FGTS,
              RES.FGTS_MES DEPOSITO_FGTS,
              (ENVIO.VRBASEFGTSMENSAL + NVL(ENVIO.VRBASEFGTS13,0)) BASE_FGTS_ESOCIAL,
              (ENVIO.VRDEPOSITOFGTS + NVL(ENVIO.VRDEPOSITOFGTS13,0)) DEPOSITO_FGTS_ESOCIAL,
              RES.FGTS - SUM(NVL(ENVIO.VRBASEFGTSMENSAL + NVL(ENVIO.VRBASEFGTS13,0), 0)) DIFERENCA_BASE,
              RES.FGTS_MES - SUM(NVL(ENVIO.VRDEPOSITOFGTS + NVL(ENVIO.VRDEPOSITOFGTS13,0), 0)) DIFERENCA_DEPOSITO
          FROM (SELECT VM.NRORG, VM.NRPESSOA,
                      VM.NRVINCULOM      VINCULO,
                      VM.DTRESCISAOVINC,
                      VM.CDMATRICULA     MATRICULA,
                      VM.DSLIVROADMISSAO REGISTRO,
                      VM.NMVINCULOM      NOME,
                      CF.DTOCORRENCIA,
                      VH.NRESTRUTLEGAL   NRLEGAL,
                      VH.NRESTRUTGEREN   NRGEREN,
                      EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                      EHL.CDCNPJESTRUT   CNPJLEGAL,
                      EML.CDINTESTRUTURA INTLEGAL,
                      EHG.NMRAZSOCESTRUT RAZAOGEREN,
                      EHG.CDCNPJESTRUT   CNPJGEREN,
                      EMG.CDINTESTRUTURA INTGEREN,
                      DECODE(ICF.NREVENTOM, 3120, 3060,
                                            3180, 3060,
                                            3210, 3060,
                                            2020, 2010,
                                            2050, 2010,
                                            2080, 2010,
                                            ICF.NREVENTOM) NREVENTOM,
                      CASE
                        WHEN ICF.NREVENTOM = 1310 THEN ICF.VRREFFOLHA
                        WHEN ICF.NREVENTOM = 20 AND NVL(ICF.VRREFFOLHA, 0) != 0 THEN ICF.VREVENTOFOLHA / ICF.VRREFFOLHA
                        ELSE ICF.VREVENTOFOLHA
                      END VREVENTOFOLHA
                  FROM GPE_VINCULOM VM
                  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                      AND VH.NRVINCULOM = VM.NRVINCULOM
                                      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                  FROM GPE_VINCULOH
                                                                WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                  AND NRORG      = VM.NRORG
                                                                  AND NRVINCULOM = VM.NRVINCULOM )
                  /*ESTRUTURA LEGAL*/
                  JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                    AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = VH.NRORG
                                          AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                          AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                      FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG = VH.NRORG
                                                                        AND NRESTRUTURAM = EML.NRESTRUTURAM )
                  /*ESTRUTURA GERENCIAL*/
                  JOIN ESTRUTURAM EMG ON EMG.NRORG = VH.NRORG
                                    AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN
                  LEFT JOIN ESTRUTURAH EHG ON EHG.NRORG = VH.NRORG
                                          AND EHG.NRESTRUTURAM = EMG.NRESTRUTURAM
                                          AND EHG.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                      FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG        = VH.NRORG
                                                                        AND NRESTRUTURAM = EMG.NRESTRUTURAM )
                  /*CÁLCULO DE FOLHA*/
                  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                          AND ICF.NRVINCULOM  = VM.NRVINCULOM
                                          AND ICF.NREVENTOM IN (3060, 2010, 3180, 2050, 3210, 2080, 3120, 2020)
                  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                          AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                          AND ( :P_NRTABELA = 1 AND CF.NRTIPOMOVIMENT IN (1, 2, 10, 19, 22)
                                                                 OR
                                                :P_NRTABELA = 4 AND CF.NRTIPOMOVIMENT IN (4)
                                                                 OR :P_NRTABELA IS NULL AND CF.NRTIPOMOVIMENT IN (1, 2, 4, 10, 19, 22) )
                                          AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                          AND CF.DTOCORRENCIA = :P_COMPETENCIA
                                          AND (CF.NROCORRECAL = :P_OCORRECAL OR :P_IGNORAOCORR = 'S')
                WHERE VM.NRORG = :P_NRORG
                  AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                  AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                  AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL OR 1 = 1)
                  AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                  AND ( INSTR(','||:P_NRESTRUTGEREN||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGEREN IS NULL )
                ) PIVOT ( SUM(VREVENTOFOLHA)
                        FOR NREVENTOM IN ( 3060 AS FGTS, 2010 AS FGTS_MES) ) RES
        
        LEFT JOIN ESO_ENVIOCADASTROS ENVIO ON ENVIO.NRORG = RES.NRORG
                                          AND TRUNC(ENVIO.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                                          AND ENVIO.NRTPAMBIENTE = 1
                                          AND ENVIO.IDATIVO = 'S' AND ENVIO.NRENVIOEXCL IS NULL 
                                          AND ENVIO.IDSTATUS IN('C','A','M')
                                          AND (ENVIO.NRTABELA = :P_NRTABELA OR :P_NRTABELA IS NULL)
        LEFT JOIN ESTRUTURAH EMP ON EMP.NRORG = ENVIO.NRORG
                                AND EMP.NRESTRUTURAM = ENVIO.NRESTRUTURAM
                                AND EMP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                            FROM ESTRUTURAH
                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                              AND NRORG        = EMP.NRORG
                                                              AND NRESTRUTURAM = EMP.NRESTRUTURAM )
        JOIN GPE_PESSOAH PE ON PE.NRPESSOA = RES.NRPESSOA 
                          AND PE.NRORG = RES.NRORG 
                          AND PE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                      FROM GPE_PESSOAH 
                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                        AND NRORG = PE.NRORG
                                                        AND NRPESSOA = PE.NRPESSOA )
        LEFT JOIN ESO_EVENTO EVESO ON EVESO.NREVENTO = ENVIO.NREVENTO
                                  AND EVESO.NMEVENTO IN ('S-2299','S-2399','S-1200')
      WHERE ((EVESO.NMEVENTO = 'S-1200' AND ENVIO.NRCHAVE = RES.NRPESSOA) OR (EVESO.NMEVENTO IN ('S-2299','S-2399') AND ENVIO.NRCHAVE = RES.VINCULO))
        AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8) AND EMP.CDCNPJESTRUT IS NOT NULL
      GROUP BY RES.VINCULO,
                RES.DTRESCISAOVINC,
                RES.MATRICULA,
                RES.REGISTRO,
                PE.NRCPFPESSOA,
                RES.NOME,
                RES.NRLEGAL,
                RES.NRGEREN,
                RES.RAZAOLEGAL,
                RES.CNPJLEGAL,
                RES.INTLEGAL,
                RES.RAZAOGEREN,
                RES.CNPJGEREN,
                RES.INTGEREN,
                ENVIO.IDSTATUS,
                ENVIO.NRTABELA,
                RES.FGTS_MES,
                RES.FGTS,
                ENVIO.VRBASEFGTSMENSAL,
                ENVIO.VRBASEFGTS13,
                ENVIO.VRDEPOSITOFGTS,
                ENVIO.VRDEPOSITOFGTS13
      )
      ORDER BY CNPJRAIZ, CNPJLEGAL, NOME
    ";
    
    static $informacoesSinteticasFgts = "
      SELECT 
      * 
      FROM (
        SELECT 
          NRLEGAL,
          RAZAOLEGAL,
          ('('||NRLEGAL||') '||RAZAOLEGAL) ESTABELECIMENTO,
          CNPJLEGAL,
          INTLEGAL,
          CNPJRAIZ,
          SUM(DEPOSITO_FGTS) DEPOSITO_FGTS,
          SUM(DEPOSITO_FGTS_ESOCIAL) DEPOSITO_FGTS_ESOCIAL,
          SUM(DEPOSITO_FGTS_13) DEPOSITO_FGTS_13,
          SUM(DEPOSITO_FGTS_ESOCIAL_13) DEPOSITO_FGTS_ESOCIAL_13,
          SUM(FGTS_RESCISAO) FGTS_RESCISAO,
          SUM( (CASE WHEN DTRESCISAOVINC IS NOT NULL THEN DEPOSITO_FGTS_ESOCIAL ELSE 0 END) ) RESCISAO_ESOCIAL,
          SUM(DIFERENCA_DEPOSITO) DIFERENCA_DEPOSITO
          FROM ( 
              SELECT RES.VINCULO,
                    RES.DTRESCISAOVINC,
                    RES.MATRICULA,
                    RES.REGISTRO,
                    PE.NRCPFPESSOA,
                    RES.NOME,
                    RES.NRLEGAL,
                    RES.NRGEREN,
                    RES.RAZAOLEGAL,
                    RES.CNPJLEGAL,
                    RES.INTLEGAL,
                    RES.RAZAOGEREN,
                    RES.CNPJGEREN,
                    RES.INTGEREN,
                    ENVIO.IDSTATUS,
                    ENVIO.NRTABELA,
                    :P_COMPETENCIA PERIODO,
                    SUBSTR(RES.CNPJLEGAL, 0,8) CNPJRAIZ,
                    SUM(RES.FGTS_MES) DEPOSITO_FGTS,
                    SUM(ENVIO.VRDEPOSITOFGTS  + NVL(ENVIO.VRDEPOSITOFGTS13,0)) DEPOSITO_FGTS_ESOCIAL,
                    SUM(RES.FGTS_MES + NVL(RES.FGTS_13,0)) - SUM(NVL(ENVIO.VRDEPOSITOFGTS  + NVL(ENVIO.VRDEPOSITOFGTS13,0),0)) DIFERENCA_DEPOSITO,
                    SUM(RES.FGTS_13) DEPOSITO_FGTS_13,
                    SUM(ENVIO.VRDEPOSITOFGTS13) DEPOSITO_FGTS_ESOCIAL_13,
                    SUM(RES.FGTS_RESCISAO) FGTS_RESCISAO
                FROM (SELECT VM.NRORG, VM.NRPESSOA,
                            VM.NRVINCULOM      VINCULO,
                            VM.DTRESCISAOVINC,
                            VM.CDMATRICULA     MATRICULA,
                            VM.DSLIVROADMISSAO REGISTRO,
                            VM.NMVINCULOM      NOME,
                            CF.DTOCORRENCIA,
                            VH.NRESTRUTLEGAL   NRLEGAL,
                            VH.NRESTRUTGEREN   NRGEREN,
                            EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                            EHL.CDCNPJESTRUT   CNPJLEGAL,
                            EML.CDINTESTRUTURA INTLEGAL,
                            EHG.NMRAZSOCESTRUT RAZAOGEREN,
                            EHG.CDCNPJESTRUT   CNPJGEREN,
                            EMG.CDINTESTRUTURA INTGEREN,
                            DECODE(ICF.NREVENTOM,   '2080', '2050', --/FGTS 13º Rescisão/
                                                    ICF.NREVENTOM) NREVENTOM,
                            CASE
                              WHEN ICF.NREVENTOM = 1310 THEN ICF.VRREFFOLHA
                              WHEN ICF.NREVENTOM = 20 AND NVL(ICF.VRREFFOLHA, 0) != 0 THEN ICF.VREVENTOFOLHA / ICF.VRREFFOLHA
                              ELSE ICF.VREVENTOFOLHA
                            END VREVENTOFOLHA
                        FROM GPE_VINCULOM VM
                        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                            AND VH.NRVINCULOM = VM.NRVINCULOM
                                            AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                        FROM GPE_VINCULOH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG      = VM.NRORG
                                                                        AND NRVINCULOM = VM.NRVINCULOM )
                        /*ESTRUTURA LEGAL*/
                        JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                          AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                        LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = VH.NRORG
                                                AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                                AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                            FROM ESTRUTURAH
                                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                              AND NRORG = VH.NRORG
                                                                              AND NRESTRUTURAM = EML.NRESTRUTURAM )
                        /*ESTRUTURA GERENCIAL*/
                        JOIN ESTRUTURAM EMG ON EMG.NRORG = VH.NRORG
                                          AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN
                        LEFT JOIN ESTRUTURAH EHG ON EHG.NRORG = VH.NRORG
                                                AND EHG.NRESTRUTURAM = EMG.NRESTRUTURAM
                                                AND EHG.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                            FROM ESTRUTURAH
                                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                              AND NRORG        = VH.NRORG
                                                                              AND NRESTRUTURAM = EMG.NRESTRUTURAM )
                      /*CÁLCULO DE FOLHA*/
                      JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                              AND ICF.NRVINCULOM = VM.NRVINCULOM
                                              AND ICF.NREVENTOM IN (2010, 2050, 2080, 2020)
                      JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                              AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                              AND ( :P_NRTABELA = 1 AND CF.NRTIPOMOVIMENT IN (1, 2, 10, 19, 22)
                                                                     OR
                                                    :P_NRTABELA = 4 AND CF.NRTIPOMOVIMENT IN (4)
                                                                     OR :P_NRTABELA IS NULL AND CF.NRTIPOMOVIMENT IN (1, 2, 4, 10, 19, 22) )
                      /*MOVIMENTO_TIPO*/
                                              AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                              AND CF.DTOCORRENCIA = :P_COMPETENCIA
                                              AND (CF.NROCORRECAL = :P_OCORRECAL OR :P_IGNORAOCORR = 'S')
                      WHERE VM.NRORG = :P_NRORG
                        AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                        AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                        AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL OR 1 = 1)
                        AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                        AND ( INSTR(','||:P_NRESTRUTGEREN||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGEREN IS NULL )
                      ) PIVOT ( SUM(VREVENTOFOLHA)
                              FOR NREVENTOM IN ( 2010 AS FGTS_MES, 2020 AS FGTS_13, 2050 AS FGTS_RESCISAO) ) RES
              
              LEFT JOIN ESO_ENVIOCADASTROS ENVIO ON ENVIO.NRORG = RES.NRORG
                                                AND TRUNC(ENVIO.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                                                AND ENVIO.NRTPAMBIENTE = 1
                                                AND ENVIO.IDATIVO = 'S' AND ENVIO.NRENVIOEXCL IS NULL 
                                                AND ENVIO.IDSTATUS IN('C','A','M')
                                                AND (ENVIO.NRTABELA = :P_NRTABELA OR :P_NRTABELA IS NULL)
              LEFT JOIN ESTRUTURAH EMP ON EMP.NRORG = ENVIO.NRORG
                                      AND EMP.NRESTRUTURAM = ENVIO.NRESTRUTURAM
                                      AND EMP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                  FROM ESTRUTURAH
                                                                  WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                    AND NRORG        = EMP.NRORG
                                                                    AND NRESTRUTURAM = EMP.NRESTRUTURAM )
              JOIN GPE_PESSOAH PE ON PE.NRPESSOA = RES.NRPESSOA 
                                AND PE.NRORG = RES.NRORG 
                                AND PE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                            FROM GPE_PESSOAH 
                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                              AND NRORG = PE.NRORG
                                                              AND NRPESSOA = PE.NRPESSOA )
              LEFT JOIN ESO_EVENTO EVESO ON EVESO.NREVENTO = ENVIO.NREVENTO
                                        AND EVESO.NMEVENTO IN ('S-2299','S-2399','S-1200')
            WHERE ((EVESO.NMEVENTO = 'S-1200' AND ENVIO.NRCHAVE = RES.NRPESSOA) OR (EVESO.NMEVENTO IN ('S-2299','S-2399') AND ENVIO.NRCHAVE = RES.VINCULO))
              AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8) AND EMP.CDCNPJESTRUT IS NOT NULL
            GROUP BY RES.VINCULO,
                      RES.DTRESCISAOVINC,
                      RES.MATRICULA,
                      RES.REGISTRO,
                      PE.NRCPFPESSOA,
                      RES.NOME,
                      RES.NRLEGAL,
                      RES.NRGEREN,
                      RES.RAZAOLEGAL,
                      RES.CNPJLEGAL,
                      RES.INTLEGAL,
                      RES.RAZAOGEREN,
                      RES.CNPJGEREN,
                      RES.INTGEREN,
                      ENVIO.IDSTATUS,
                      ENVIO.NRTABELA,
                      RES.FGTS_MES,
                      RES.FGTS_RESCISAO,
                      RES.FGTS_13,
                      RES.FGTS_RESCISAO,
                      ENVIO.VRDEPOSITOFGTS,
                      ENVIO.VRDEPOSITOFGTS13
            )
    GROUP BY NRLEGAL, RAZAOLEGAL, CNPJLEGAL, INTLEGAL, CNPJRAIZ)
    ORDER BY CNPJRAIZ, CNPJLEGAL
    ";
    
    static $informacoesSobreBasesCalculoFgts = "
      SELECT * FROM ( 
        SELECT 
            NOME, 
            NRCPFPESSOA,
            NRLEGAL,
            RAZAOLEGAL,
            ('('||NRLEGAL||') '||RAZAOLEGAL||' '||CNPJLEGAL) ESTABELECIMENTO,
            CNPJLEGAL,
            PRI.CDTERCEIRO,
            CNPJRAIZ,
            PRI.FGTS BASE,
            (CASE WHEN PRI.DTRESCISAOVINC IS NULL THEN PRI.VRBASEFGTSMENSAL ELSE 0 END) BASE_ESOCIAL,
            PRI.FGTS_13 BASE_13,
            (CASE WHEN PRI.DTRESCISAOVINC IS NULL THEN PRI.VRBASEFGTS13 ELSE 0 END) BASE_13_ESOCIAL,
            PRI.FGTS_RESCISAO,
            (CASE WHEN PRI.DTRESCISAOVINC IS NOT NULL THEN PRI.VRBASEFGTSMENSAL + NVL(PRI.VRBASEFGTS13,0) ELSE 0 END) RESCISAO_ESOCIAL,
            PRI.NRESTRUTLOTACAO,
            ( '('||PRI.NRESTRUTLOTACAO||') '||'FPAS '||PRI.CDFPAS||' '||'Terceiros '||PRI.CDTERCEIRO ) LOTACAO
          FROM (SELECT 
                    VM.NRVINCULOM VINCULO,
                    PE.NRCPFPESSOA,
                    VM.NMVINCULOM NOME,
                    VH.NRESTRUTLEGAL NRLEGAL,
                    EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                    EHL.CDCNPJESTRUT CNPJLEGAL,
                    SUBSTR(EHL.CDCNPJESTRUT, 0,8) CNPJRAIZ,
                    ENVIO.NRESTRUTLOTACAO,
                    LOT.CDFPAS,
                    LOT.CDTERCEIRO,
                    DECODE(ICF.NREVENTOM,   '3060', '2010', --/FGTS/
                                            '3180', '2050', --/FGTS Rescisão/
                                            '3120', '2020', --/INCID_FGTS_13SAL - FGTS Décimo Terceiro/
                                            '3210', '2050', --/FGTS 13º Rescisão/
                                            ICF.NREVENTOM) NREVENTOM,
                    CASE
                      WHEN ICF.NREVENTOM = 1310 THEN ICF.VRREFFOLHA
                      WHEN ICF.NREVENTOM = 20 AND NVL(ICF.VRREFFOLHA, 0) != 0 THEN ICF.VREVENTOFOLHA / ICF.VRREFFOLHA
                      ELSE ICF.VREVENTOFOLHA
                    END VREVENTOFOLHA,
                    ENVIO.VRBASEFGTSMENSAL,
                    ENVIO.VRBASEFGTS13,
                    VM.DTRESCISAOVINC
                  FROM GPE_VINCULOM VM
                  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                      AND VH.NRVINCULOM = VM.NRVINCULOM
                                      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                                WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                  AND NRORG      = VM.NRORG
                                                                  AND NRVINCULOM = VM.NRVINCULOM )
                  JOIN GPE_PESSOAH PE ON PE.NRPESSOA = VM.NRPESSOA 
                                    AND PE.NRORG = VM.NRORG 
                                    AND PE.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH 
                                                              WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                AND NRORG = PE.NRORG
                                                                AND NRPESSOA = PE.NRPESSOA)
                  /*ESTRUTURA LEGAL*/
                  JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                          AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG        = VH.NRORG
                                          AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                          AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = VH.NRORG
                                                                      AND NRESTRUTURAM = EML.NRESTRUTURAM )
                  JOIN ESO_ENVIOCADASTROS ENVIO ON ENVIO.NRORG = VM.NRORG
                                                    AND TRUNC(ENVIO.DTMESCOMPETENC, 'MM') = :P_COMPETENCIA
                                                    AND ENVIO.NRTPAMBIENTE = 1
                                                    AND ENVIO.IDATIVO = 'S' AND ENVIO.NRENVIOEXCL IS NULL 
                                                    /*MOVIMENTO_ESOCIAL*/
                                                    AND ENVIO.IDSTATUS IN('C','A','M')
                                                    AND (ENVIO.NRTABELA = :P_NRTABELA OR :P_NRTABELA IS NULL)
                  JOIN ESO_EVENTO EVESO ON EVESO.NREVENTO = ENVIO.NREVENTO
                                            AND EVESO.NMEVENTO IN ('S-2299','S-2399','S-1200')
                  LEFT JOIN ESTRUTURAH EMP ON EMP.NRORG        = ENVIO.NRORG
                                          AND EMP.NRESTRUTURAM = ENVIO.NRESTRUTURAM
                                          AND EMP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = EMP.NRORG
                                                                      AND NRESTRUTURAM = EMP.NRESTRUTURAM )
                  LEFT JOIN ESTRUTURAH LOT ON LOT.NRORG        = ENVIO.NRORG
                                          AND LOT.NRESTRUTURAM = ENVIO.NRESTRUTLOTACAO
                                          AND LOT.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = LOT.NRORG
                                                                      AND NRESTRUTURAM = LOT.NRESTRUTURAM )
                  LEFT JOIN CNAEFISCAL CF ON CF.CDCNAE = LOT.CDCNAE
                  LEFT JOIN (SELECT PEM.NRORG, PEM.NRESTRUTURAM, MAX(DECODE(ITG.NREVENTOM,3590,ITG.VRPERCENT,0)) VRPERCEMPRESA, MAX(DECODE(ITG.NREVENTOM,3870,ITG.VRPERCENT,0)) VRPERCTERCEIROS
                              FROM GPE_PARAMESTRUTM PEM
                              JOIN GPE_PARAMESTRUTH PEH ON PEH.NRORG = PEM.NRORG
                                                        AND PEH.NRPARAMESTRUTM = PEM.NRPARAMESTRUTM
                                                        AND PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PARAMESTRUTH H1
                                                                                    WHERE H1.DTMESCOMPETENC <= :P_COMPETENCIA
                                                                                      AND H1.NRORG = PEH.NRORG
                                                                                      AND H1.NRPARAMESTRUTM = PEH.NRPARAMESTRUTM )
                              JOIN FPA_PARAMFOLHAM PFM ON PFM.NRORG = PEH.NRORG
                                                      AND PFM.NRPARAMFOLHAM = PEH.NRPARAMFOLHAM
                              JOIN FPA_PARAMFOLHAH PFH ON PFH.NRORG = PFM.NRORG
                                                      AND PFH.NRPARAMFOLHAM = PFM.NRPARAMFOLHAM
                                                      AND PFH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_PARAMFOLHAH H1
                                                                                  WHERE H1.DTMESCOMPETENC <= :P_COMPETENCIA
                                                                                    AND H1.NRORG = PFH.NRORG
                                                                                    AND H1.NRPARAMFOLHAM = PFH.NRPARAMFOLHAM )
                              JOIN FPA_GRPINCIDE GRI ON GRI.NRORG = PFH.NRORG
                                                    AND GRI.NRGRPINCIDE = PFH.NRGRPINCIDE
                              JOIN FPA_GRPITEM ITG ON ITG.NRORG = GRI.NRORG
                                                  AND ITG.NRGRPINCIDE = GRI.NRGRPINCIDE
                              WHERE PEM.NRORG = :P_NRORG
                              GROUP BY PEM.NRORG, PEM.NRESTRUTURAM) PER ON PER.NRORG = LOT.NRORG
                                                                      AND PER.NRESTRUTURAM = LOT.NRESTRUTURAM

                  /*CÁLCULO DE FOLHA*/
                  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                          AND ICF.NRVINCULOM = VM.NRVINCULOM
                                          AND ICF.NREVENTOM IN (3060, 3180, 3210, 3120)
                  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                          AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                          AND ( :P_NRTABELA = 1 AND CF.NRTIPOMOVIMENT IN (1, 2, 10, 19, 22)
                                                                 OR
                                                :P_NRTABELA = 4 AND CF.NRTIPOMOVIMENT IN (4)
                                                                 OR :P_NRTABELA IS NULL AND CF.NRTIPOMOVIMENT IN (1, 2, 4, 10, 19, 22) )
                  /*MOVIMENTO_TIPO*/
                                          AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                          AND CF.DTOCORRENCIA = :P_COMPETENCIA
                                          AND (CF.NROCORRECAL = :P_OCORRECAL OR :P_IGNORAOCORR = 'S')

                  WHERE VM.NRORG = :P_NRORG
                    AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                    AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                    AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL )
                    AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                    AND ((EVESO.NMEVENTO = 'S-1200' AND ENVIO.NRCHAVE = VM.NRPESSOA) OR (EVESO.NMEVENTO IN ('S-2299','S-2399') AND ENVIO.NRCHAVE = VM.NRVINCULOM))
                    AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(EHL.CDCNPJESTRUT,1,8) AND EMP.CDCNPJESTRUT IS NOT NULL
                  GROUP BY 
                    VM.NRVINCULOM,
                    PE.NRCPFPESSOA,
                    VM.NMVINCULOM,
                    VH.NRESTRUTLEGAL,
                    EHL.NMRAZSOCESTRUT,
                    EHL.CDCNPJESTRUT,
                    ENVIO.VRBASE13,
                    ENVIO.DTMESCOMPETENC,
                    SUBSTR(EHL.CDCNPJESTRUT, 0,8),
                    ENVIO.NRESTRUTLOTACAO,
                    LOT.CDFPAS,
                    LOT.CDTERCEIRO,
                    ICF.NREVENTOM,
                    ICF.VREVENTOFOLHA,
                    ICF.VRREFFOLHA,
                    ENVIO.VRBASEFGTSMENSAL,
                    ENVIO.VRBASEFGTS13,
                    VM.DTRESCISAOVINC
                    ) PIVOT ( SUM(VREVENTOFOLHA)
                                FOR NREVENTOM IN ( 2010 AS FGTS, 2020 AS FGTS_13, 2050 AS FGTS_RESCISAO ) ) PRI
      GROUP BY NOME, NRCPFPESSOA, NRLEGAL, RAZAOLEGAL, CNPJLEGAL, PRI.CDFPAS, PRI.CDTERCEIRO, CNPJRAIZ, PRI.FGTS, PRI.NRESTRUTLOTACAO, PRI.VRBASEFGTSMENSAL, PRI.VRBASEFGTS13, PRI.FGTS_RESCISAO, PRI.FGTS_13, PRI.DTRESCISAOVINC )
      ORDER BY CNPJRAIZ, CNPJLEGAL, NOME
            ";
}
