<?php
namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\{ Repositories, Util, HCMParameters, DateUtil, Messages, DoctrineEntityUtil };
use HCM\PTO\Services\{ OcorrenciaFrequenciaService, ApuracaoFrequenciaService };
use HCM\Geral\Repository\VinculoRepository;

class RoteiroUsoFrequenciaService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, DoctrineEntityUtil $doctrineEntityUtil) {
        $this->entityManager            = $entityManager;
        $this->hcmParameters            = $hcmParameters;
        $this->doctrineEntityUtil       = $doctrineEntityUtil;
    }
    
    /**
     *  Rotina responsavel por atualizar a data de ultima operacao (DTULTOPER) de um determinado passo de uso de frequencia (PTO_PASSOSUSOFREQ). Essa informacao eh armazenada em PTO_PERIODOUSOFREQ
     * 
     *  @param  $nrPassosUsoFreq
     *  @param  $nrPeriodoApuracao
     * 
     *  @return bool
     * 
     */
    public function setUsoFreqDataUltOper(
        $nrPassosUsoFreq,
        $nrPeriodoApuracao
    ){
       
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
       
        $existingPeriodoPasso = $this->entityManager->getRepository(Repositories::PTO_PERIODOUSOFREQ)->findOneBy(array('nrorg' => $nrOrg, 'nrpassosusofreq' => $nrPassosUsoFreq, 'nrperiodoapuracao' => $nrPeriodoApuracao));
        if (!empty($existingPeriodoPasso)) {
            $existingPeriodoPasso->setDtultoper(new \DateTime);
            $existingPeriodoPasso->setDtultatu(new \DateTime);
            $existingPeriodoPasso->setNrorgultatu($nrOrg);
            $existingPeriodoPasso->setCdoperultatu($this->hcmParameters->getCdOperador());
            
            $this->entityManager->persist($existingPeriodoPasso);
            $this->entityManager->flush();
            return true;
        }
       
        $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_PERIODOUSOFREQ,
            [
                'NRPERIODOUSOFREQ'      => \HCM\Util\NovoCodigo::geraCodigo($nrOrg, 'PTO_PERIODOUSOFREQ', 1, 12, true),
                'NRPASSOSUSOFREQ'       => $nrPassosUsoFreq,
                'NRPERIODOAPURACAO'     => $nrPeriodoApuracao,
                'DTULTOPER'             => new \DateTime,
            ],
            true,
            true,
            true
        );
        
        return true;
    }
    
} 