<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 01/03/2016
 * Time: 11:14
 */

namespace HCM\VAB\Services;

use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use HCM\FPA\Services\ExportacaoExcelService;
use HCM\Util\Repositories;
use Folha\Entities\VabTarifa;

class LinhatarifaService {

    protected $hcmParameters;

    public function __construct(Kernel $httpKernel,HCMParameters $hcmParameters,EntityManager $entitymanager, FPA $fpa, ExportacaoExcelService $exportacaoExcelService) {
        $this->httpKernel = $httpKernel;
        $this->entityManager = $entitymanager;
        $this->hcmParameters = $hcmParameters;
        $this->fpa = $fpa;
        $this->exportacaoExcelService = $exportacaoExcelService;
    }
    
    public function baixarPlanilhaLinhatarifa($nrorg, $nmarquivo){
        $cabecalho = array(
            "Nr. Linha",
            "Nr. fornecedor",
            "Descrição da linha",
            "Descrição da tarifa",
            "Valor da Tarifa",
            "Forma de transporte",
            "Operação de Transporte",
            "Categoria do Transporte",
            "Tipo de Transporte",
        );
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        $exemplo = array();

        $exemplo[0][] = "1058";
        $exemplo[0][] = "1";
        $exemplo[0][] = "ABC PARAIBA - TAUBATÉ MUNICIPAL - ELETRNICO";
        $exemplo[0][] = "TARIFA 3,90";
        $exemplo[0][] = "3.90";
        $exemplo[0][] = "1";
        $exemplo[0][] = "1";
        $exemplo[0][] = "1";
        $exemplo[0][] = "1";

        return $this->exportacaoExcelService->gravarTemplate_1($nrorg, $cabecalho, $nmarquivo, $exemplo);
    }
    
    public function importarPlanilhaLinhatarifa($row) {
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); 
        
        $head = array(
            'NRLINHATRANSP',
            'CDFORNECED',
            'DSLINHATRANSP',
            'DSTARIFA',
            'VRTARIFA',
            'NRFORMATRANSPORTE',
            'NROPERTRANSPORTE',
            'NRCATEGTRANSPORTE',
            'NRTIPOTRANSPORTE'
            );
        
        $file = $this->httpKernel->factoryFromFileData($row['DSPATH'][0], FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);

        $dados = $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('NRLINHATRANSP'));

        foreach($dados as $lin => $dado) {
            $res = $this->insereLinhatarifa($lin + 2, $nrorg, $nrorgPadrao, $dado);
            foreach($res as $msg)
                $messages[] = $msg;
        }
        return $messages;
    }
    
     
    public function insereLinhatarifa($row, $nrorg, $nrorgPadrao, $dado){
        $msg = array();
        
        //Substitui a virgula do VRTARIFA por um ponto caso seja necessario
        $dado['VRTARIFA'] = is_string($dado['VRTARIFA']) ? str_replace(",",".",$dado['VRTARIFA']) : $dado['VRTARIFA'];
        
        //Teste de existencia do campo
        if($dado['NRLINHATRANSP'] == null){ $msg[] = array("msg" => "O número da linha é obrigatorio!", "row" => $row); }
        if($dado['CDFORNECED'] == null){ $msg[] = array("msg" => "O codigo do fornecedor é obrigatorio!", "row" => $row); }
        if($dado['DSLINHATRANSP'] == null){ $msg[] = array("msg" => "A descrição de linha é obrigatoria!", "row" => $row); }
        if($dado['DSTARIFA'] == null){ $msg[] = array("msg" => "A descrição de tarifa é obrigatoria!", "row" => $row); }
        if($dado['VRTARIFA'] == null){ $msg[] = array("msg" => "O valor de tarifa é obrigatoria!", "row" => $row); }
        if($dado['NRFORMATRANSPORTE'] == null){ $msg[] = array("msg" => "O número da forma de transporte é obrigatoria!", "row" => $row); }
        if($dado['NROPERTRANSPORTE'] == null){ $msg[] = array("msg" => "O número da operação de transporte é obrigatorio!", "row" => $row); }
        if($dado['NRCATEGTRANSPORTE'] == null){ $msg[] = array("msg" => "O número da categoria de transporte é obrigatoria", "row" => $row); }
        if($dado['NRTIPOTRANSPORTE'] == null){ $msg[] = array("msg" => "O número do tipo de transporte é obrigatorio", "row" => $row); }
        
        //Teste de comprimento de campo
        if(strlen( $dado['CDFORNECED'])>21){ $msg[] = array("msg" => "O codigo do fornecedor ultrapassa o limite de 21 caracteres", "row" => $row); }
        if(strlen( $dado['DSLINHATRANSP'])>100){ $msg[] = array("msg" => "A descrição de linha informada ultrapassa o limite de 100 caracteres", "row" => $row); }
        if(strlen( $dado['DSTARIFA'])>50){ $msg[] = array("msg" => "A descrição de tarifa informada ultrapassa o limite de 100 caracteres", "row" => $row); }
        
        //Teste para saber se o campo é um valor numerico
        if(!is_numeric($dado['NRLINHATRANSP'])){ $msg[] = array("msg" => "O número de linha informado não é um número", "row" => $row); }
        if(!is_numeric($dado['VRTARIFA'])){ $msg[] = array("msg" => "O valor de tarifa informado não é um número", "row" => $row); }
        if(!is_numeric($dado['NRFORMATRANSPORTE'])){ $msg[] = array("msg" => "O número da forma de transporte informado não é um número", "row" => $row); }
        if(!is_numeric($dado['NROPERTRANSPORTE'])){ $msg[] = array("msg" => "O número da operação de transporte informado não é um número", "row" => $row); }
        if(!is_numeric($dado['NRCATEGTRANSPORTE'])){ $msg[] = array("msg" => "O número da categoria de transporte informado não é um número", "row" => $row); }
        if(!is_numeric($dado['NRTIPOTRANSPORTE'])){ $msg[] = array("msg" => "O número do tipo de transporte informado não é um número", "row" => $row); }
        
        //Teste de existencia no Banco de Dados
         if(count($msg) == 0){
            
            $obj = $this->entityManager->getRepository(Repositories::VAB_LINHA)->findOneBy(array('nrorg'=>$nrorg,'nrlinhatransp'=>$dado['NRLINHATRANSP']));
            if($obj){ $msg[] = array("msg" => "O número da linha informada " .$dado['NRLINHATRANSP']." já existe!", "row" => $row); }
            
            $obj = $this->entityManager->getRepository(Repositories::FORNECEDOR)->findOneBy(array('nrorg'=>$nrorg,'cdforneced'=>$dado['CDFORNECED']));
            if(!$obj){ $msg[] = array("msg" => "O codigo do fornecedor " .$dado['CDFORNECED']." não foi encontrado no Banco de Dados!", "row" => $row); }
            
            $obj = $this->entityManager->getRepository(Repositories::FORMATRANSPORTE)->findOneBy(array('nrorg' => array( $nrorg, $nrorgPadrao),'nrformatransporte'=>$dado['NRFORMATRANSPORTE']));
            if(!$obj){ $msg[] = array("msg" => "O numero da forma de transporte informada " .$dado['NRFORMATRANSPORTE']." não foi encontrado no Banco de Dados!", "row" => $row); }
            
            $obj = $this->entityManager->getRepository(Repositories::OPERTRANSPORTE)->findOneBy(array('nrorg' => array( $nrorg, $nrorgPadrao),'nropertransporte'=>$dado['NROPERTRANSPORTE']));
            if(!$obj){ $msg[] = array("msg" => "O numero da operação de transporte informada " .$dado['NROPERTRANSPORTE']." não foi encontrado no Banco de Dados!", "row" => $row); }
        
            $obj = $this->entityManager->getRepository(Repositories::CATEGTRANSPORTE)->findOneBy(array('nrorg' => array( $nrorg, $nrorgPadrao),'nrcategtransporte'=>$dado['NRCATEGTRANSPORTE']));
            if(!$obj){ $msg[] = array("msg" => "O numero da categoria de transporte informada " .$dado['NRCATEGTRANSPORTE']." não foi encontrado no Banco de Dados!", "row" => $row); }
            
            $obj = $this->entityManager->getRepository(Repositories::TIPOTRANSPORTE)->findOneBy(array('nrorg' => array( $nrorg, $nrorgPadrao),'nrtipotransporte'=>$dado['NRTIPOTRANSPORTE']));
            if(!$obj){ $msg[] = array("msg" => "O numero do tipo de transporte informada " .$dado['NRTIPOTRANSPORTE']." não foi encontrado no Banco de Dados!", "row" => $row); }
        }
        
        if(count($msg) == 0){
            $nrVabTarifa = $this->insereVabTarifa( $dado['DSTARIFA'], $dado['VRTARIFA'],$this->hcmParameters->getDtCompetencia());
            $nrVabLinha = $this->insereVabLinha( $dado['NRLINHATRANSP'], $dado['CDFORNECED'], $dado['NRFORMATRANSPORTE'], $dado['NROPERTRANSPORTE'], $dado['NRCATEGTRANSPORTE'], $dado['NRTIPOTRANSPORTE'], $dado['DSLINHATRANSP'], $this->hcmParameters->getDtCompetencia());
            $this->insereVabLinhaTarifa($nrVabLinha, $nrVabTarifa, $this->hcmParameters->getDtCompetencia());
        }
        
        return $msg;
            
    }
    
    public function insereVabTarifa($dsTarifa, $vrTarifa, $dtInitarifa){
        $repVabTarifa = Repositories::VAB_TARIFA;
        $objTarifa = new $repVabTarifa;
        
        $nrtarifa = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'VAB_TARIFA', 1, 12, true);
        
        $objTarifa->setDstarifa($dsTarifa);
        $objTarifa->setVrtarifa($vrTarifa);
        $objTarifa->setDtinitarifa(DateUtil::getDataDeString($dtInitarifa, DateUtil::FORMATO_BRASILEIRO, true));
        $objTarifa->setIdativo('S');
        $objTarifa->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $objTarifa->setDtinclusao(DateUtil::getDataAtual());
        $objTarifa->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $objTarifa->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $objTarifa->setDtultatu(DateUtil::getDataAtual());
        $objTarifa->setCdoperultatu($this->hcmParameters->getCdoperador());
        $objTarifa->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $objTarifa->setNrtarifa($nrtarifa);
        
        $this->entityManager->persist($objTarifa);
        $this->entityManager->flush($objTarifa);
        
        return $nrtarifa;
    }
    
    public function insereVabLinha( $nrLinhatransp, $cdForneced, $nrFormatransporte, $nrOpertransporte, $nrCategtransporte, $nrTipotransporte, $dsLinhatransp, $dtInilinha){
        $repVabLinha = Repositories::VAB_LINHA;
        $objLinha = new $repVabLinha; 
        
        $objLinha->setNrlinhatransp($nrLinhatransp);
        $objLinha->setCdforneced($cdForneced);
        $objLinha->setNrformatransporte($nrFormatransporte);
        $objLinha->setNropertransporte($nrOpertransporte);
        $objLinha->setNrcategtransporte($nrCategtransporte);
        $objLinha->setNrtipotransporte($nrTipotransporte);
        $objLinha->setDslinhatransp($dsLinhatransp);
        $objLinha->setDtinilinha(DateUtil::getDataDeString($dtInilinha, DateUtil::FORMATO_BRASILEIRO, true));
        $objLinha->setIdativo('S');
        $objLinha->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $objLinha->setDtinclusao(DateUtil::getDataAtual());
        $objLinha->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $objLinha->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $objLinha->setDtultatu(DateUtil::getDataAtual());
        $objLinha->setCdoperultatu($this->hcmParameters->getCdoperador());
        $objLinha->setNrorg($this->hcmParameters->getNrorgTrabalho());
        
        $this->entityManager->persist($objLinha);
        $this->entityManager->flush($objLinha);
        
        return $nrLinhatransp;
    }
    
    public function insereVabLinhaTarifa($nrLinhatransp, $nrTarifa, $dtInilinhatarifa){
        
        $repVabLinhaTarifa = Repositories::VAB_LINHATARIFA;
        $objLinhaTarifa = new $repVabLinhaTarifa; 
        
        $nrlinhatarifa = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_LINHATARIFA', 1, 12, true);
        
        $objLinhaTarifa->setNrlinhatarifa($nrlinhatarifa);          
        $objLinhaTarifa->setNrlinhatransp($nrLinhatransp);      
        $objLinhaTarifa->setNrtarifa($nrTarifa);             
        $objLinhaTarifa->setDtinilinhatarifa(DateUtil::getDataDeString($dtInilinhatarifa, DateUtil::FORMATO_BRASILEIRO, true));    
        $objLinhaTarifa->setIdativo('S');                   
        $objLinhaTarifa->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());                
        $objLinhaTarifa->setDtinclusao(DateUtil::getDataAtual());                       
        $objLinhaTarifa->setCdoperinclusao($this->hcmParameters->getCdoperador());            
        $objLinhaTarifa->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());              
        $objLinhaTarifa->setDtultatu(DateUtil::getDataAtual());                   
        $objLinhaTarifa->setCdoperultatu($this->hcmParameters->getCdoperador());              
        $objLinhaTarifa->setNrorg($this->hcmParameters->getNrorgTrabalho());           
        
        $this->entityManager->persist($objLinhaTarifa);
        $this->entityManager->flush($objLinhaTarifa);
        
        return $nrlinhatarifa;
    }
}
