<?php
namespace Folha\Logic\ESocial\Eventos\Data;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-2220 - Alteração de Dados Cadastrais do
 * Trabalhador
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S2220
{

    /**
     * Retorna os dados que serao utilizados para a geracao do(s) arquivo(s) do evento S-2220, Alteração de Dados
     * Cadastrais do Trabalhador.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo)
    {
        if ($versaoEsocial === '2.0') {
            $S2220 = \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S2220(
                $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo
            );
        }
        else {
            $S2220 = \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S2220_v11(
                $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                $utilizaIntervaloDeDatas, $nrorg, $versaoHRM
            );

            foreach ($S2220 as &$row) {
                /*Parte criada para setar as chaves pais que não possuem valor, são do tipo G, para que no xml o IF do TWIG reconheça que existem as chaves filhas (elementos)*/
                if (isset($row['NRCTPS']) or isset($row['SERIECTPS']) or isset($row['UFCTPS']) or
                    isset($row['NRRG']) or isset($row['RGORGAOEMISSOR']) or isset($row['RGDTEXPEDICAO']) or
                    isset($row['NRRNE']) or
                    isset($row['NRCNH']) or isset($row['CNHDTEXPEDICAO']) or isset($row['CNHDTVALIDADE'])
                ) {

                    $row['DOCUMENTOS'] = '1';

                    /*CTPS*/
                    if (isset($row['NRCTPS']) or isset($row['SERIECTPS']) or isset($row['UFCTPS'])) {
                        $row['DOCUMENTOS_CTPS'] = '1';
                    }

                    /*RG*/
                    if (isset($row['NRRG']) or isset($row['RGORGAOEMISSOR']) or isset($row['RGDTEXPEDICAO'])) {
                        $row['DOCUMENTOS_RG'] = '1';
                    }

                    /*RNE*/
                    if (isset($row['NRRNE'])) {
                        $row['DOCUMENTOS_RNE'] = '1';
                    }

                    /*CNH*/
                    if (isset($row['NRCNH']) or isset($row['CNHDTEXPEDICAO']) or isset($row['CNHDTVALIDADE'])) {
                        $row['DOCUMENTOS_CNH'] = '1';
                    }
                }

                /*Aqui são gerados os dependentes do empregado em um sub array no xml o FOR do TWIG possa buscar os dados para cada empregado*/
                $dependentes = \Folha\Repositories\GpeEsocialRegistro::s2100_Dependentes($row['NRPARCNEGOCIO'], $nrorg);
                $contDep = 1;
                foreach ($dependentes as &$dependente) {
                    $row['DEPENDENTE'][$contDep] =
                        array("_EVTCADINICIAL_TRABALHADOR_DEPENDENTE_TPDEP" => $dependente['TPDEP'],
                            "_EVTCADINICIAL_TRABALHADOR_DEPENDENTE_NOMEDEP" => $dependente['NMPESSOA'],
                            "_EVTCADINICIAL_TRABALHADOR_DEPENDENTE_DTNASCTO" => $dependente['DTNASCPESSOA'],
                            "_EVTCADINICIAL_TRABALHADOR_DEPENDENTE_CPFDEP" => $dependente['NRCPFPESSOA'],
                            "_EVTCADINICIAL_TRABALHADOR_DEPENDENTE_DEPIRRF" => $dependente['DEPIRRF'],
                            "_EVTCADINICIAL_TRABALHADOR_DEPENDENTE_DEPSF" => $dependente['DEPSF']);
                    $contDep++;
                }
            }
        }

        return $S2220;
    }

}